//+------------------------------------------------------------------+
//|                                     static_Objects_inception.mq4 |
//|                                                      nicholishen |
//|                                   www.reddit.com/u/nicholishenFX |
//+------------------------------------------------------------------+
#property copyright "nicholishen"
#property link      "www.reddit.com/u/nicholishenFX"
#property version   "1.00"
#property strict
#include <Arrays\ArrayInt.mqh>

class MyClass : public CArrayInt
{
protected:
   // static master class instance (clone of the original instance). 
   // Holds the original collection set by OnInit.
   static MyClass*   m_origin;
   // number of class instances running
   static int        m_instances;
public:
   MyClass() 
   { 
      m_instances++; 
      Print("MyClass instances = ",m_instances);
   }
  ~MyClass() 
   {   
      // ONLY delete the static instance when we are destroying the last remaining object.
      if(m_instances <= 2 && CheckPointer(m_origin)==POINTER_DYNAMIC && &this != m_origin)
         delete m_origin;
      Print("Destroy instance ",m_instances);
      m_instances--;   
   }
   void  operator = (MyClass &in) { this = &in; }  // This is NOT assignment! Call to...
   void  operator = (MyClass *in)                  // ...this overloaded operator!
   { 
      AssignArray(in);
   }
   
   void OnInit(CArrayInt* in) // Initializes the object once and all instances inherit this collection.
   {
      this.AssignArray(in);
      if(m_origin==NULL)
         m_origin = new MyClass; 
      m_origin.AssignArray(&this);
   }
   void FilterByInt(int element)
   {
      if(Total() == 0)
         this = m_origin;     // Again, NOT assignment. Calling = operator function.
      for(int i=Total()-1;i>=0;i--)
         if(this[i]!=element)
            Delete(i);
   }
   MyClass* FilterReset()
   {
      this = m_origin;
      return &this;
   }
};
MyClass* MyClass::m_origin = NULL;
int MyClass::m_instances = 0;
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
{ 
   srand(GetTickCount());
   CArrayInt int_arr;
   for(int i=0;i<100;i++)
      int_arr.Add(rand()%10+1);
   
   MyClass *origin, other, diff;
   
   origin = new MyClass();
   origin.OnInit(&int_arr);
   delete origin;
   //origin is detroyed, but it's clone still exists in the static object.
  
   //"diff" and "other" have not been explicitly assigned any values. But they clone the collection from m_origin
   diff.FilterByInt(4);
   other = diff;
   other.FilterByInt(7); //will result in 0 elements since diff only contains 4's'
   Print("other.Total() = ",other.Total());
   for(int i=0;i<diff.Total();i++)
      Print("diff[",i,"] = ",diff[i]);
  
}
//+------------------------------------------------------------------+
